<?php
// $Id: uc_clickandbuy_recurring.ca.inc,v 1.1.2.1 2010/02/17 14:55:46 longwave Exp $

/**
 * Implementation of hook_ca_entity().
 */
function uc_clickandbuy_recurring_ca_entity() {
  $entities['uc_clickandbuy_subscription'] = array(
    '#title' => t('ClickandBuy subscription event type'),
    '#type' => 'string',
  );

  return $entities;
}

/**
 * Implementation of hook_ca_condition().
 */
function uc_clickandbuy_recurring_ca_condition() {
  $conditions['uc_clickandbuy_recurring_event_type'] = array(
    '#title' => t('Check the subscription event type'),
    '#category' => t('ClickandBuy'),
    '#callback' => 'uc_clickandbuy_recurring_condition_event_type',
    '#arguments' => array(
      'event' => array('#entity' => 'uc_clickandbuy_subscription'),
    ),
  );

  return $conditions;
}

/**
 * Implementation of hook_ca_action().
 */
function uc_clickandbuy_recurring_ca_action() {
  $actions['uc_clickandbuy_recurring_grant_role'] = array(
    '#title' => t('Grant a role on an order.'),
    '#category' => t('ClickandBuy'),
    '#callback' => 'uc_clickandbuy_recurring_action_grant_role',
    '#arguments' => array(
      'order' => array(
        '#entity' => 'uc_order',
        '#title' => t('Order'),
      ),
    ),
  );

  $actions['uc_clickandbuy_recurring_revoke_role'] = array(
    '#title' => t('Revoke a role on an order.'),
    '#category' => t('ClickandBuy'),
    '#callback' => 'uc_clickandbuy_recurring_action_revoke_role',
    '#arguments' => array(
      'order' => array(
        '#entity' => 'uc_order',
        '#title' => t('Order'),
      ),
    ),
  );

  return $actions;
}

/**
 * Implementation of hook_ca_trigger().
 */
function uc_clickandbuy_recurring_ca_trigger() {
  $triggers['uc_clickandbuy_subscription'] = array(
    '#title' => t('ClickandBuy subscription event'),
    '#category' => t('ClickandBuy'),
    '#arguments' => array(
      'order' => array(
        '#entity' => 'uc_order',
        '#title' => t('Order'),
      ),
      'event' => array(
        '#entity' => 'uc_clickandbuy_subscription',
        '#title' => t('ClickandBuy subscription event type'),
      ),
    ),
  );

  return $triggers;
}

/**
 * Check a subscription event type.
 */
function uc_clickandbuy_recurring_condition_event_type($event, $settings) {
  return in_array($event, array_filter($settings['event']));
}

function uc_clickandbuy_recurring_condition_event_type_form($form_state, $settings = array()) {
  $form['event'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Event types'),
    '#options' => array(
      'activated' => 'Subscription activated',
      'recurring' => 'Subscription renewed',
      'blocked' => 'Subscription blocked',
      'unblocked' => 'Subscription unblocked',
      //'ended' => 'Subscription ended',
      'terminated' => 'Subscription terminated',
    ),
    '#default_value' => is_array($settings['event']) ? $settings['event'] : array(),
  );

  return $form;
}

/**
 * Grant a role.
 */
function uc_clickandbuy_recurring_action_grant_role($order, $settings) {
  $rid = $settings['role'];
  $roles = user_roles(TRUE);

  if ($roles[$rid]) {
    $account = user_load($order->uid);
    $account->roles += array($rid => $roles[$rid]);
    user_save($account, array('roles' => $account->roles));
    watchdog('uc_clickandbuy', 'Granted !role role to !user', array('!role' => $roles[$rid], '!user' => $account->name));
  }
}

function uc_clickandbuy_recurring_action_grant_role_form($form_state, $settings = array()) {
  $roles = user_roles(TRUE);
  unset($roles[DRUPAL_AUTHENTICATED_RID]);

  if (empty($roles)) {
    drupal_set_message(t('There are no roles available to select. <a href="!url">Add a role</a>.', array('!url' => url('admin/user/roles'))), 'error');
  }
  
  $form['role'] = array(
    '#type' => 'select',
    '#title' => t('Role'),
    '#options' => $roles,
    '#default_value' => $settings['role'],
  );

  return $form;
}

/**
 * Revoke a role.
 */
function uc_clickandbuy_recurring_action_revoke_role($order, $settings) {
  $rid = $settings['role'];
  $roles = user_roles(TRUE);

  if ($roles[$rid]) {
    $account = user_load($order->uid);
    unset($account->roles[$rid]);
    user_save($account, array('roles' => $account->roles));
    watchdog('uc_clickandbuy', 'Revoked !role role from !user', array('!role' => $roles[$rid], '!user' => $account->name));
 }
}

function uc_clickandbuy_recurring_action_revoke_role_form($form_state, $settings = array()) {
  return uc_clickandbuy_recurring_action_grant_role_form($form_state, $settings);
}
